# -----------------------------------------------------------------------------
package admin_groups;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Mar 21: 
#          
# -- 
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

use admin_bits;

use config;
use debug;
use exceptions;

use Passwd;

# -----------------------------------------------------------------------------

require Exporter;

@ISA    = qw(Exporter);
@EXPORT = qw(print_add_group_form 
             add_group
             select_remove_groups_users
             remove_groups_users
             actually_remove_groups_users
             select_add_groups_users
             add_groups_users
             actually_add_groups_users);

# -----------------------------------------------------------------------------
# -- MODULE/PACKAGE VARS :
# -----------------------------------------------------------------------------

my $root         = &root;
my $passwd_dir   = &passwd_dir;
my $html_lib_dir = &html_lib_dir;

# -----------------------------------------------------------------------------
# -- EXPORTS :

# -- 

sub print_add_group_form() {

  my $TREE     = shift;
  my $USERNAME = shift;

    my $passwd = Passwd->new;
    my $groups = $passwd->{GROUPS};

    open (FORM, "$html_lib_dir/admin_add_group_form.html_src") 
        || &raise_stop_exception("Can't open admin_add_group_form.html_src");

    my $textline;

    while ($textline = <FORM>) {
        $textline =~ s/%tree%/$TREE/;
        $textline =~ s/%username%/$USERNAME/;
        $textline =~ s/%groups%/@$groups/;
        print $textline;
      }

    close (FORM);
  }


# -----------------------------------------------------------------------------
# -- add a group to one of the existing group files :

sub add_group () {

  my $TREE       = shift;
  my $USERNAME   = shift;
  my $NEWGROUP   = shift;
  my $NEWGRPCOMM = shift;
    
##    print "\n<BR> t: $TREE";
##    print "\n<BR> u: $USERNAME";
##    print "\n<BR> g: $NEWGROUP";
##    print "\n<BR> c: $NEWGRPCOMM";

    open(GRP, ">>$passwd_dir/_G_some_groups")
        || &raise_stop_exception("Can't open group-group-file " . 
                                 "(for appending)");

    print GRP "\n\n\# $NEWGRPCOMM";
    print GRP "\n$NEWGROUP";

    close(GRP);

    # -- tell 'em :

    my $title_refresh = &refresh_to_admin_menu_alt_too($TREE, $USERNAME,
                         "Added group!");

    print $title_refresh;

    print "<TABLE>" .
          "<TR><TD><B>Group:     </B></TD>" . 
          "<TD><TT>$NEWGROUP</TT></TD></TR></TABLE>\n";

    print "<BR><BR>N.B. <I>There are no users in your group yet!</I>";
  }

# -----------------------------------------------------------------------------
# --

sub select_remove_groups_users() {

  my $USERNAME = shift;
  my $TREE     = shift;

print "\n<BR> u: $USERNAME";
print "\n<BR> t: $TREE";

    my ($textline, $group_selector);

    my $passwd = Passwd->new;
    my $groups = $passwd->{GROUPS};
  
    $group_selector = "<SELECT NAME=\"SELGROUP\" SIZE=1>\n";
    foreach (@$groups) {$group_selector .= "<OPTION>$_\n";}
    $group_selector .= "</SELECT>\n"; 

    open (FORM, "$html_lib_dir/admin_group_select_form.html_src")
        || &raise_stop_exception("Can't open " . 
                                 "admin_group_select_form");

    while($textline = <FORM>) {
        $textline =~ s/%username%/$USERNAME/;
        $textline =~ s/%tree%/$TREE/;
        $textline =~ s/%agaction%/removegroupsusers/;
        $textline =~ s/%groupselector%/$group_selector/;
        print $textline;
      }

    close (FORM);
  }


# -----------------------------------------------------------------------------
# --

sub select_add_groups_users() {

  my $USERNAME = shift;
  my $TREE     = shift;

print "\n<BR> u: $USERNAME";
print "\n<BR> t: $TREE";

    my ($textline, $group_selector);

    my $passwd = Passwd->new;
    my $groups = $passwd->{GROUPS};
  
    $group_selector = "<SELECT NAME=\"SELGROUP\" SIZE=1>\n";
    foreach (@$groups) {$group_selector .= "<OPTION>$_\n";}
    $group_selector .= "</SELECT>\n"; 

    open (FORM, "$html_lib_dir/admin_group_select_form.html_src")
        || &raise_stop_exception("Can't open " . 
                                 "admin_group_select_form");

    while($textline = <FORM>) {
        $textline =~ s/%username%/$USERNAME/;
        $textline =~ s/%tree%/$TREE/;
        $textline =~ s/%agaction%/addgroupsusers/;
        $textline =~ s/%groupselector%/$group_selector/;
        print $textline;
      }

    close (FORM);
  }


# -----------------------------------------------------------------------------
# -- 

sub remove_groups_users() {

  my $USERNAME = shift;
  my $TREE     = shift;
  my $GROUP    = shift;

print "\n<BR> u: $USERNAME";
print "\n<BR> t: $TREE";

    my $passwd       = Passwd->new();
    my $passwd_files = $passwd->{PASSWDFILES};

    my ($pf);

    print "\n<FORM NAME=\"removegroupsusers\" METHOD=\"POST\" " . 
          "ACTION=\"admin_groups.pl\">";

    foreach $pf (@$passwd_files) {

        my $pf_group_users = $passwd->passwd_file_group_users($pf, $GROUP);

        foreach (@$pf_group_users) {
            print "\n<BR> $_: <INPUT TYPE=\"checkbox\" NAME=\"REMOVE\" " . 
                  "VALUE=\"$pf/$_\">";
          }
      }
    print "\n<BR><INPUT TYPE=\"submit\" VALUE=\"Enter...\">";

    print "\n<INPUT TYPE=\"hidden\" NAME=\"USERNAME\" VALUE=\"$USERNAME\">";
    print "\n<INPUT TYPE=\"hidden\" NAME=\"TREE\" VALUE=\"$TREE\">";
    print "\n<INPUT TYPE=\"hidden\" NAME=\"GROUP\" VALUE=\"$GROUP\">";
    print "\n<INPUT TYPE=\"hidden\" NAME=\"AGACTION\" " . 
          "VALUE=\"actuallyremovegroupsusers\">";

    print "\n</FORM>";
  }


# -----------------------------------------------------------------------------
# -- 

sub add_groups_users() {

  my $USERNAME = shift;
  my $TREE     = shift;
  my $GROUP    = shift;

print "\n<BR> u: $USERNAME";
print "\n<BR> t: $TREE";

    my $passwd       = Passwd->new();
    my $passwd_files = $passwd->{PASSWDFILES};

    my ($pf);

    print "\n<FORM NAME=\"addgroupsusers\" METHOD=\"POST\" " . 
          "ACTION=\"admin_groups.pl\">";

    foreach $pf (@$passwd_files) {

        my $pf_group_users_not = 
            $passwd->passwd_file_group_users_not($pf, $GROUP);

        foreach (@$pf_group_users_not) {
            print "\n<BR> $_: <INPUT TYPE=\"checkbox\" NAME=\"ADD\" " . 
                  "VALUE=\"$pf/$_\">";
          }
      }
    print "\n<BR><INPUT TYPE=\"submit\" VALUE=\"Enter...\">";

    print "\n<INPUT TYPE=\"hidden\" NAME=\"USERNAME\" VALUE=\"$USERNAME\">";
    print "\n<INPUT TYPE=\"hidden\" NAME=\"TREE\" VALUE=\"$TREE\">";
    print "\n<INPUT TYPE=\"hidden\" NAME=\"GROUP\" VALUE=\"$GROUP\">";
    print "\n<INPUT TYPE=\"hidden\" NAME=\"AGACTION\" " . 
          "VALUE=\"actuallyaddgroupsusers\">";

    print "\n</FORM>";
  }


# -----------------------------------------------------------------------------
# -- 

sub actually_remove_groups_users() {

  my $TREE            = shift;
  my $USERNAME        = shift;
  my $GROUP           = shift;
  my $GRP_USR_RM_LIST = shift;

print "\n<BR> u: $USERNAME";
print "\n<BR> t: $TREE";

    my ($passwd_file, $username);

    my $passwd = Passwd->new();

    foreach (@$GRP_USR_RM_LIST) {
        print "\n<BR> gonna remove /$_/";
        ($passwd_file, $username) = split(/\//, $_);

        $passwd->remove_group_user($GROUP, $username, $passwd_file);
      }

    # -- tell 'em :

    my $title_refresh = &refresh_to_admin_menu_alt_too($TREE, $USERNAME,
                         "Removed the following users:");

    print $title_refresh;

    foreach (@$GRP_USR_RM_LIST) {
        ($passwd_file, $username) = split(/\//, $_);
        print "\n<BR> $passwd_file: $username";
      }
  }


# -----------------------------------------------------------------------------
# -- 

sub actually_add_groups_users() {

  my $TREE            = shift;
  my $USERNAME        = shift;
  my $GROUP           = shift;
  my $GRP_USR_ADD_LIST = shift;

print "\n<BR> u: $USERNAME";
print "\n<BR> t: $TREE";

    my ($passwd_file, $username);

    my $passwd = Passwd->new();

    foreach (@$GRP_USR_ADD_LIST) {
        print "\n<BR> gonna add /$_/";
        ($passwd_file, $username) = split(/\//, $_);

        $passwd->add_group_user($GROUP, $username, $passwd_file);
      }

    # -- tell 'em :

    my $title_refresh = &refresh_to_admin_menu_alt_too($TREE, $USERNAME,
                         "Added the following users:");

    print $title_refresh;

    foreach (@$GRP_USR_ADD_LIST) {
        ($passwd_file, $username) = split(/\//, $_);
        print "\n<BR> $passwd_file: $username";
      }
  }


# -----------------------------------------------------------------------------
# -- PRIVATES :



# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
