# -----------------------------------------------------------------------------
package Passwd;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Mar 09: 
#          
# -- 2001 Mar 26: -- quick code review, commenting(!)...
#
#                 -- integrated (old, traditional module) passwd.pm into 
#                    Passwd.pm (with consequences for navigate.pl and
#                    nav_mod.pm);
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

use config;
use Net::LDAP;

# -----------------------------------------------------------------------------
# -- PUBLIC INTERFACE :
#    
#    new --- constructor;
#    
#    {PASSWDFILES}, {USERNAMES}, {GROUPS} --- pointers to lists of 
#        currently existing password-files, usernames and user-groups;
#    
#    ok_username, ok_groups, ok_passwd_file, ok_password --- are the given
#        fields ok for creation of new user?
#    
#    add_group_user, remove_group_user, passwd_file_group_users_not
#    passwd_file_group_users --- library routines to add and remove users
#        to/from a group, get users in a particular group, get users not
#        in a particular group;


# -----------------------------------------------------------------------------
# -- MODULE/PACKAGE VARS :

my $cgi_root_dir = &cgi_root_dir;
my $tree         = &tree;
my $root         = &root;
my $passwd_dir   = &passwd_dir;

# -----------------------------------------------------------------------------
# -- PUBLIC CONSTRUCTOR :
# -----------------------------------------------------------------------------

sub new () {

  my $class  = shift;     
  my $except = shift;

    my $self  = { };              

    bless($self, $class);             

    $self->{PASSWDFILES} = &current_passwd_files($except);
    $self->{USERNAMES}   = &current_usernames($except, $self->{PASSWDFILES});
    $self->{GROUPS}      = &current_groups($except);

    $self->{PASSWD}      = &current_Passwd($except);

    return $self; 
  }


# -----------------------------------------------------------------------------
# -- PUBLIC :  (ADMIN) LIBRARY ROUTINES :
# -----------------------------------------------------------------------------

# -- add a user from a group :

sub add_group_user() {

  my $self        = shift;
  my $except      = shift;
  my $group       = shift;
  my $username    = shift;
  my $passwd_file = shift;

    my ($u, $p, $r, $gg);
    my (@g);

    open(PASSWD, "$passwd_dir/$passwd_file")
        || $except->raise_stop("Can't open passwd file: $passwd_file");

    my @textlines = ();
    my $textline  = "";

    # -- search through all the lines in the password file to find the one
    #    we want :

    while ($textline = <PASSWD>) {

        # -- unless this line matches the given username :

        unless ($textline =~ m/^\s*$username\s*&/) {
            push(@textlines, $textline); 
            next;
          }

        chomp($textline);
        $textline =~ s/\s*$//;
        $textline .= " " . $group . " \n";
        push(@textlines, $textline); 
      }
    close(PASSWD);

    # -- overwrite the passwd-file with the altered version :
    open(PASSWD, ">$passwd_dir/$passwd_file")
        || $except->raise_integrity("unable to open passwd file : " . 
                                      "$passwd_file");

    foreach (@textlines) {print PASSWD $_}

    close(PASSWD);
  }

# -----------------------------------------------------------------------------
# -- remove a user from a group :

sub remove_group_user() {

  my $self        = shift;
  my $except      = shift;
  my $group       = shift;
  my $username    = shift;
  my $passwd_file = shift;

    my ($u, $p, $r, $gg);
    my (@g);

    open(PASSWD, "$passwd_dir/$passwd_file")
        || $except->raise_stop("Can't open passwd file: $passwd_file");

    my @textlines = ();
    my $textline  = "";

    # -- search through all the lines in the password file to find the one
    #    we want :

    while ($textline = <PASSWD>) {

        # -- unless this line matches the given username :

        unless ($textline =~ m/^\s*$username\s*&/) {
            push(@textlines, $textline); 
            next;
          }
 
        ($u, $p, $r, $gg) = split(/\s+&\s+/, $textline);

        @g = split(/\s/, $gg);
        $gg = "";
        foreach(@g) {unless($_ eq $group) {$gg .= $_ . " "}}

        $textline = "$u & $p & $r & $gg\n";
        push(@textlines, $textline); 
     }
    close(PASSWD);

    # -- overwrite the passwd-file with the filtered version :
    open(PASSWD, ">$passwd_dir/$passwd_file")
        || $except->raise_integrity("unable to open passwd file : " . 
                                      "$passwd_file");

    foreach (@textlines) {print PASSWD $_}

    close(PASSWD);
  }


# -----------------------------------------------------------------------------
# -- get users NOT in a particular group from a particular passwd file :

sub passwd_file_group_users_not() {

    my $self        = shift;
    my $except      = shift;
    my $passwd_file = shift;
    my $group       = shift;

    open(PASSWD, "$passwd_dir/$passwd_file")
        || $except->raise_stop("Can't open password file: $passwd_file");

    my $textline;
    my ($u, $p, $r, $gg);
    my (@g, @u);

    while ($textline = <PASSWD>) {
 
        # -- if not a blank line and not a comment :
        if ($textline =~ m/\S/) {
            unless ($textline =~ m/^\s*\#/) {

                ($u, $p, $r, $gg) = split(/&/, $textline);  

                $u =~ s/\s*//g;  
                @g = split(/\s/, $textline);

                my $in_group = 0;

                foreach (@g) {
                    if ($group eq $_) {$in_group = 1}
                  }

                unless ($in_group) {push(@u, $u);}
              }
          }
      }
    close(PASSWD);

    return \@u;
  }


# -----------------------------------------------------------------------------
# -- get users in a particular group from a particular passwd file :

sub passwd_file_group_users() {

    my $self        = shift;
    my $except      = shift;
    my $passwd_file = shift;
    my $group       = shift;

    open(PASSWD, "$passwd_dir/$passwd_file")
        || $except->raise_stop("Can't open password file: $passwd_file");

    my $textline;
    my ($u, $p, $r, $gg);
    my (@g, @u);

    while ($textline = <PASSWD>) {
 
        # -- if not a blank line and not a comment :
        if ($textline =~ m/\S/) {
            unless ($textline =~ m/^\s*\#/) {

                ($u, $p, $r, $gg) = split(/&/, $textline);  

                $u =~ s/\s*//g;  
                @g = split(/\s/, $textline);

                my $in_group = 0;

                foreach (@g) {
                    if ($group eq $_) {$in_group = 1}
                  }

                if ($in_group) {push(@u, $u);}
              }
          }
      }
    close(PASSWD);

    return \@u;
  }


# -----------------------------------------------------------------------------
# -- PUBLIC :  LIBRARY ROUTINES :
# -----------------------------------------------------------------------------

# -- wander through all password files to see if given uname and password
#    are valid;  if not found there, try LDAP :

sub valid_uname_password {

  my $self     = shift @_;
  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $username = shift @_;
  my $password = shift @_;

    # -- TRY LOCAL : ----------------------------------------------------------

    # -- vars associated with extracting possible matches to the given
    #    username/password from our password files :
    #
    my ($filename, $textline, $u, $p, $r, $salt, $password_crypt);

    # -- look in our password-file-directory for password files;  look in
    #    each in turn :

    opendir(PASSWDDIR, $passwd_dir) 
        || $except->raise_logout("passwd_dir_unavail"); 

    foreach $filename (@{$self->{PASSWDFILES}}) {

        # -- open our password file;  read in username/password combinations 
        #    from our password file :

        open(PASSWDFILE, "$passwd_dir/$filename") 
            || $except->raise_logout('passwd_file_unavail'); 

        while ($textline = <PASSWDFILE>) {

            # -- get username and password from text-line;  trim off any 
            #    whitespace;  and try them against our client's attempt :

            ($u, $p, $r) = split(/&/, $textline);  
            $u =~ s/\s*//g;  
            $p =~ s/\s*//g;            

            if ($username eq $u) {
                $salt = crypt($password, "pepper");
                $password_crypt = crypt($password, $salt);
                if ($password_crypt eq $p) {return 1;}
              } 

          } # ...while looking through password file;
        close(PASSWDFILE);

      } # ...foreach password file in password directory;
    closedir(PASSWDDIR);

    
    # -- TRY LDAP : -----------------------------------------------------------

    my $ldap     = Net::LDAP->new('padl.umist.ac.uk');
    my $ret_mesg = $ldap->bind("cn=$username,ou=isd,o=umist", 
                               password=>$password);

    if ($ret_mesg->code()) {
#        $except->raise_logout($cookies, $doc, 
#                              "Unable to bind to LDAP server.");
      }
    else {
        return 1;
      }

#    my $ret_mesg 
#            = $ldap->search(base=>'ou=isd,o=umist', scope=>'sub',
#                            filter=>"&(objectClass=person)(uid=$username)");
#
#    my $entry      = $ret_mesg->shift_entry();
#    my @attributes = $entry->attributes();
#
#    foreach my $a (@attributes) { 
#        my @att_vals = $entry->get_value($a);
#        foreach (@att_vals) {print "\n $_"}
#      }
 
    return 0;
  }


# -----------------------------------------------------------------------------
# --

sub in_group() {

  my $self     = shift;
  my $except   = shift;
  my $username = shift;
  my $group    = shift;

    $group =~ s/^GROUP_//;

    my ($filename, $textline);
    my ($u, $p, $r, $g, $gg, @g);

    opendir(PASSWDDIR, $passwd_dir) 
        || $except->raise_logout("passwd_dir_unavail"); 

    foreach $filename (@{$self->{PASSWDFILES}}) {

        open(PASSWDFILE, "$passwd_dir/$filename") 
            || $except->raise_logout('passwd_file_unavail'); 

        while ($textline = <PASSWDFILE>) {

            ($u, $p, $r, $gg) = split(/&/, $textline);  
            $u                =~ s/\s*//g;  
            @g                = split(/\s/, $gg);

            if ($username eq $u) {
                foreach (@g) {
                    if ($group eq $_) { return 1; }
                  }
              } 

          } # ...while looking through password file;
        close(PASSWDFILE);

      } # ...foreach password file in passwrod directory;
    closedir(PASSWDDIR);

    return 0;
  }


# -----------------------------------------------------------------------------

# --

sub realname() {

  my $self     = shift;
  my $except   = shift;
  my $username = shift @_;

    my ($filename, $realname, $textline, $u, $p);

    opendir(PASSWDDIR, $passwd_dir) 
        || $except->raise_logout("passwd_dir_unavail"); 

    foreach $filename (@{$self->{PASSWDFILES}}) {

        open(PASSWDFILE, "$passwd_dir/$filename") 
            || $except->raise_logout('passwd_file_unavail'); 

        while ($textline = <PASSWDFILE>) {

            ($u, $p, $realname) = split(/&/, $textline);  

            $u =~ s/\s*//g;  

            if ($u eq $username) {
                $realname =~ s/^\s*//;
                $realname =~ s/\s*$//;
                $realname =~ s/\s/\~/g;
                return $realname;
	      }
	  }
      }
  }



# -----------------------------------------------------------------------------
# -- PUBLIC :  INPUT FIELD-VALIDITY ROUTINES :
#    
#    Check fields for creating a new user.
#    
#    Some sneeky individual might try sending us naughty strings in an 
#    attempt to get a look at (or write over/to) files they shouldn't.
#    These routines attempt to check that does not happen...
#    
#    Also...see individual routine notes.
# -----------------------------------------------------------------------------

# -- is the username received valid --- does it consist of nice sensible
#    characters;  also, does the username already exist?

sub ok_username() {

  my $self     = shift;
  my $username = shift;

    # -- only letters or underscores :
    if ($username =~ m/[^a-zA-Z\_]/) {return 0}

    # -- check if exists already :
    foreach (@{$self->{USERNAMES}}) {
        if ($_ eq $username) {return 0}
      }
    return 1;
  }

# -----------------------------------------------------------------------------
# -- are the received groups valid --- do they already exist?

sub ok_groups() {

  my $self   = shift;
  my $groups = shift;

    my @groups = split(/\s+/, $groups);
    my $found  = 0;

    my $group;

    foreach $group (@groups) {

        # -- ensure exists already :
        $found = 0;

        foreach (@{$self->{GROUPS}}) {
            if ($_ eq $group) {
                $found = 1;
              }
          }
        unless ($found) {return 0}
      }
    return 1;
  }

# -----------------------------------------------------------------------------
# -- does the received passwd file actually exist?

sub ok_passwd_file() {

  my $self        = shift;
  my $passwd_file = shift;

    # -- ensure exists already :

    foreach (@{$self->{PASSWDFILES}}) {
        if ($_ eq $passwd_file) {
            return 1;
          }
      }
    return 0;
  }

# -----------------------------------------------------------------------------
# -- is the password received an ok one, e.g., is it long enough?

sub ok_password() {

  my $self     = shift;
  my $password = shift;
                                                         # ------ TODO ! ------
    return 1;                                            # ------ TODO ! ------
  }


# -----------------------------------------------------------------------------
# -- LOCAL :  CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------


# -- return a pointer to a hash of "everything" (except (encrypted) 
#    passwords)) in the PASSWD dir :

sub current_Passwd() {
    
    my $except  = shift;

    my $filename;
    my $textline;
    my ($u, $p, $r, $gg);

    my $passwd_files = { };

    opendir(PASSWDDIR, $passwd_dir) 
        || $except->raise_logout("passwd_dir_unavail"); 

    foreach $filename (sort readdir(PASSWDDIR)) {

        # -- skip past junk :
        if ($filename eq ".") {next};
        if ($filename eq "..") {next};
        if ($filename =~ m/[^~]*~/) {next};

        # -- skip past this (is it actually any use?) :
        if ($filename =~ m/^_G_/) {next};

        my $passwd_file = { };
            # ...hash keyed on usernames representing this password file;

        # -- everything else is assumed to be a password file which consists
        #    of rows of 
        #       <username> & <encry passwd> & <realname> & <group> <group>...

        open(PASSWDFILE, "$passwd_dir/$filename") 
            || $except->raise_logout("passwd_file_unavail");

        while ($textline = <PASSWDFILE>) {
            ($u, $p, $r, $gg) = split(/&/, $textline);  

            unless ($u =~ m/\S/) {next;}

            my $entry = { };

            $u  =~ s/\s*//g;  
            $p  =~ s/\s*//g;  
            $gg =~ s/^\s*//;    $gg =~ s/\s*$//;    

            my @gg = split(/\s/, $gg);
####print "\n<BR>UNAME: $u";
            $entry->{UNAME} = $u;
            $entry->{RNAME} = $r;
            $entry->{GRPS}  = \@gg;

            $passwd_file->{$u} = $entry;
          }        
        $passwd_files->{$filename} = $passwd_file;
      }

    closedir(PASSWDDIR);

    return $passwd_files;
        # ...return pointer to hash of password files, keyed on password file
        #    name, in which the values are pointers to hashes keyed on 
        #    usernames the attributes of which are UNAME, RNAME, GRPS, 
        #    where the latter is a pointer to a list of all their groups...
  }


# -----------------------------------------------------------------------------
# -- get list of currently existing password files :

sub current_passwd_files() {

  my $except = shift;

    my @passwd_files = ();
    my $filename;


    opendir(PASSWDDIR, $passwd_dir) 
        || $except->raise_logout("passwd_dir_unavail"); 

    foreach $filename (sort readdir(PASSWDDIR)) {

        if ($filename eq ".") {next};
        if ($filename eq "..") {next};
        if ($filename =~ m/[^~]*~/) {next};
        if ($filename =~ m/^_G_/) {next};

        push(@passwd_files, $filename);
      }
    closedir(PASSWDDIR);

    return \@passwd_files;
  }


# -----------------------------------------------------------------------------
# -- get list of currently existing usernames from all password files :

sub current_usernames() {

  my $except       = shift;
  my $passwd_files = shift;

    my ($filename, $textline);
    my ($u, $p, $r, $gg);
    my @usernames;

    opendir(PASSWDDIR, $passwd_dir) 
        || $except->raise_logout("passwd_dir_unavail"); 

    foreach $filename (@{$passwd_files}) {

        open(PASSWDFILE, "$passwd_dir/$filename") 
            || $except->raise_logout("passwd_file_unavail");

        while ($textline = <PASSWDFILE>) {
            if ($textline =~ m/\S/) {
                ($u, $p, $r, $gg) = split(/&/, $textline);  
                $u =~ s/\s*//g;  
                push(@usernames, $u);
              }
          }
        close(PASSWDFILE);
      }
    closedir(PASSWDDIR);

    return \@usernames;
  }


# -----------------------------------------------------------------------------
# -- get list of currently existing user-groups :

sub current_groups() {

  my $except = shift;

    my ($filename, $textline);
    my (%groups, @groups, @g);

    opendir(PASSWDDIR, $passwd_dir) 
        || $except->raise_logout("passwd_dir_unavail"); 

    foreach $filename (sort readdir(PASSWDDIR)) {

        if ($filename =~ m/[^~]*~/) {next};
        unless ($filename =~ m/^_G_/) {next};

        open(GRPFILE, "$passwd_dir/$filename") 
            || $except->raise_logout('passwd_file_unavail');

        while ($textline = <GRPFILE>) {
            if ($textline =~ m/\S/) {

                if ($textline =~ m/^\s*\#/) {next}
                unless ($textline =~ m/\S/) {next}
                    # ...comments, blank lines;

                @g = split(/\s/, $textline);

                foreach (@g) {
                    $_ =~ s/\s*//g;
                    $groups{$_} = 1;
                  }
              }
          }
        close(GRPFILE);
      }   

    @groups = sort keys %groups;
    return \@groups;
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
