# -----------------------------------------------------------------------------

package Tree;

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Mar 09: 
#          
# -- 2001 Mar 26: -- quick code review, commenting(!)...
#
# -- 2001 Jun 19: -- moved "existing" subroutines from thingies.pm to
#                    here and changed some to methods, some to part of
#                    constructor --- probably best if all are methods...

# -----------------------------------------------------------------------------

use strict;
use config;

use Thingy;


# -----------------------------------------------------------------------------
# -- Module/Package variables :

my $root = &root;


# -----------------------------------------------------------------------------
# -- PUBLIC FIELDS :
#    
#    {TREE}     --- pointers to children/branches in a Tree (Entries are 
#                   ignored);
#
#    {TREETOO}  --- pointers to children/branches in a Tree, also giving
#        permissions and Readme (Entries are ignored);  this includes the 
#        root as a "special" child/branch...
#
#    {AUTHORS}  --- all authors in the tree
#    {KEYWORDS} --- all keywords in the tree


# -----------------------------------------------------------------------------
# -- PUBLIC :  CONSTRUCTOR :
# -----------------------------------------------------------------------------


# -- returns what's in a Tree in terms of CDFs, ignorming Entries :

sub new {                        

  my $class  = shift;           
  my $except = shift;
  my $tree   = shift;

    my $self = { };              

    my $textline;

    bless($self, $class);             

    $self->{NAME}    = $tree;
    $self->{TREE}    = &new_tree($tree, "/");
    $self->{TREETOO} = &new_tree_too($tree, "/");

    $self->{AUTHORS}  = &existing_authors($except, $tree);
    $self->{KEYWORDS} = &existing_keywords($except, $tree);

    return $self;
  }


# -----------------------------------------------------------------------------
# -- PUBLIC :  RETURN METHODS :
# -----------------------------------------------------------------------------

# -- return pointer to hash representing tree structure (CDF's are atomic 
#    in this) :

##sub tree () {
##    my $self = shift;
##    return $self->{TREE};
##  }


# -----------------------------------------------------------------------------
# -- returns all THINGYs in a Tree (recursively) :

my @ex_thingies;

sub existing_thingies() {

  my $self   = shift;
  my $except = shift;

    &get_existing_thingies($except, $self->{NAME}, "/");
    return \@ex_thingies;
  }

sub get_existing_thingies() {

  my $except   = shift;
  my $tree     = shift;
  my $location = shift;

    my @all_results = ();
    my $thingy;
    my $file;

    opendir(DIR, "$root$tree$location") 
        || $except->raise_logout("Cannot open directory: " .
                                 "\"$root$tree$location\" " .
                                 "in order to " .
                                 "find Entries.");

    foreach $file (readdir(DIR)) {

        if (($file eq ".") || ($file eq "..")) {next;}

        # -- it's a directory :
        if (-d "$root$tree$location$file") {
            &get_existing_thingies($except, $tree, $location . $file . "/");
          }

        # -- its a plain file :
        if (-f "$root$tree$location$file") {

            if ($file =~ m/\-\-THINGY$/) {             

                $file =~ s/\-\-THINGY$//;
                $thingy = Thingy->down_new($tree, $location, $file);
                    
                my $result = { };
                $result->{LOCATION} = $location;
                $result->{FILE}     = $file;
                $result->{THINGY}   = $thingy;

                push(@ex_thingies, $result);
              }
          }
      }

    closedir(DIR);

    return \@ex_thingies;
  }


# -----------------------------------------------------------------------------
# -- LOCAL :  CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------

# -- recursively search tree to determine full structure in terms of CDFs,
#    ignoring entries;  store structure in a hash (of hashes (of hashes...)) :

sub new_tree () {
  my $tree     = shift;
  my $location = shift;

    my @children = ();
    my $child;

    opendir(TREE, "$root/$tree/$location")
        || die "Can't open DIR: $root/$tree/$location";

    foreach $child (sort readdir(TREE)) {
        if ($child eq ".") {next}
        if ($child eq "..") {next}

        open(FILE, "$root$tree$location/$child") 
            || die "Can't open FILE: $root$tree$location$child";

        if (-d FILE) {
            push (@children, "$location/$child");
            my $childs = &new_tree($tree, "$location/$child");
            push (@children, @$childs);
          }
        else {
            close(FILE);
            next;
          }
      }

    closedir(TREE);
    return \@children;
  }

# -----------------------------------------------------------------------------

sub new_tree_too () {
  my $tree     = shift;
  my $location = shift;

    my @children = ();
    my $child;
    my $textline;
    my ($readme);

    opendir(TREE, "$root/$tree/$location")
        || die "Can't open DIR: $root/$tree/$location";

    foreach $child (sort readdir(TREE)) {
        if ($child eq ".") {next}
        if ($child eq "..") {next}

        open(FILE, "$root$tree$location/$child") 
            || die "Can't open FILE: $root$tree$location$child";

        if (-d FILE) {

            my $ref = { };

            open (README, "$root$tree$location/$child/README.node")  
                || die "Can't open: $root$tree$location/$child/README.node";
            my $readme = "";
            while ($textline = <README>) {
                $readme .= $textline;
              }
            close (README);

            open (ALLOW, "$root$tree$location/$child/USERS.ALLOW")  
                || die "Can't open: $root$tree$location/$child/USERS.ALLOW";
            my $allow = "";
            while ($textline = <ALLOW>) {
                if ($textline =~ m/^\s*\#/) {next}
                unless ($textline =~ m/\S/) {next}
                $allow .= $textline;
              }
            close (ALLOW);

            open (DENY, "$root$tree$location/$child/USERS.DENY")  
                || die "Can't open: $root$tree$location/$child/USERS.DENY";
            my $deny = "";
            while ($textline = <DENY>) {
                if ($textline =~ m/^\s*\#/) {next}
                unless ($textline =~ m/\S/) {next}
                $deny .= $textline;
              }
            close (DENY);

            $ref->{CDF} = "$location/$child";
            $ref->{README} = $readme;
            $ref->{ALLOW}  = $allow;
            $ref->{DENY}   = $deny;
            push (@children, $ref);
            my $childs = &new_tree_too($tree, "$location/$child");
            push (@children, @$childs);
          }
        else {
            close(FILE);
            next;
          }
      }

    closedir(TREE);

    return \@children;
  }


# -----------------------------------------------------------------------------
# -- construct AUTHORS field :

sub existing_authors () {

    my $except  = shift;
    my $tree    = shift;
    my $authors = { };

    &get_existing_authors($except, $tree, "/", $authors);

    return $authors;
  }


sub get_existing_authors () {

  my $except   = shift;
  my $tree     = shift;
  my $location = shift;
  my $auths    = shift;

    opendir(DIR, "$root$tree$location") 
        || $except->raise_logout("Cannot open directory: " .
                                 "\"$root$tree$location\" " .
                                 "in order to " .
                                 "determine current list of authors.");
    
    foreach (readdir(DIR)) {

        if (($_ eq ".") || ($_ eq "..")) {next;}

        # -- it's a directory :
        if (-d "$root$tree$location$_") {
            &get_existing_authors($except, 
                                  $tree, $location . $_ . "/", $auths);
          }

        # -- its a plain file :
        if (-f "$root$tree$location$_") {

            if ($_ =~ m/\-\-THINGY$/) {    
                $_ =~ s/\-\-THINGY$//;
                my $thingy = Thingy->down_new($tree, $location, $_);
                my $auth   = $thingy->author;

                $auths->{$auth} = 1;
              }
          }
      }   

    closedir(DIR);
  }


# -----------------------------------------------------------------------------
# -- construct KEYWORDS field :

sub existing_keywords() {

    my $except   = shift;
    my $tree     = shift;
    my $keywords = { };

    &get_existing_keywords($except, $tree, "/", $keywords);

    return $keywords;
  }

# -- gets all the keywords associated with all the ENTRYs in the CDF specified
#    and all the CDFs below (recursively) :

sub get_existing_keywords() {

  my $except   = shift;
  my $tree     = shift;
  my $location = shift;
  my $keyws    = shift;

    my $thingy;
    my $file;

    opendir(DIR, "$root$tree$location") 
        || &raise_logout_exception("Cannot open directory: " .
                                   "\"$root$tree$location\" " .
                                   "in order to " .
                                   "determine current list of keywords.");
    
    foreach $file (readdir(DIR)) {

        if (($file eq ".") || ($file eq "..")) {next;}

        # -- it's a directory :
        if (-d "$root$tree$location$file") {
            &get_existing_keywords($except, 
                                   $tree, $location . $file . "/", $keyws);
          }

        # -- its a plain file :
        if (-f "$root$tree$location$file") {

            if ($file =~ m/\-\-THINGY$/) {             
                $file =~ s/\-\-THINGY$//;
                $thingy = Thingy->down_new($tree, $location, $file);
                my $kws = $thingy->keywords;
                foreach (@$kws) {$keyws->{$_} = 1}
              }
          }
      }

    closedir(DIR);
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
