# -----------------------------------------------------------------------------
package thingies;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2000 Jan 24: 
#          
#    helper-module for Thingy: offers things like author lists, keyword lists,
#    searches...
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

# -----------------------------------------------------------------------------

require Exporter;

# -----------------------------------------------------------------------------

use Time::Local;

use config;
use Thingy;

# -----------------------------------------------------------------------------

@ISA    = qw(Exporter);
@EXPORT = qw(thingy_exists
             author_filter
             date_filter
             grep_entry_filter
             grep_filenames_filter
             );

# -----------------------------------------------------------------------------

my $root         = &root;
my $cgi_root_dir = &cgi_root_dir;

# -----------------------------------------------------------------------------
# -- ALL THINGIES :
# -----------------------------------------------------------------------------

# -- does a document with this name-root exist already?

sub thingy_exists() {

  my $except   = shift;
  my $tree      = shift @_;
  my $location  = shift @_;
  my $name_root = shift @_;

    my ($entry, @entries);

    opendir(DIR, $root . $tree . $location) 
        || $except->raise_integrity("Can't open directory: " . 
                                      "$root . $tree . $location");

    while ($entry = readdir(DIR)) {
        @entries = split(/--/, $entry);
        if ($name_root eq $entries[0]) {close(DIR);  return 1}
      }
    close(DIR);
    return 0;
  }


# -----------------------------------------------------------------------------
# FILTERS :
# -----------------------------------------------------------------------------

# --

sub grep_entry_filter {

  my $results = shift;
  my $grep    = shift;

    my @filtered_results = ();

    foreach (@$results) {
        if ($_->{FILE} =~ m/$grep/i) {
            push(@filtered_results, $_);
          }
      }

    return \@filtered_results;
  }


# -----------------------------------------------------------------------------
# --

sub grep_filenames_filter {

  my $results = shift;
  my $grep    = shift;

    my @filtered_results = ();

    foreach (@$results) {
        if ( ($_->{THINGY}->{DEFINITIVE}[0] =~ m/$grep/i) || 
             ($_->{THINGY}->{VIEWONLY}[0] =~ m/$grep/i)) {
            push(@filtered_results, $_);
          }
      }

    return \@filtered_results;
  }


# -----------------------------------------------------------------------------
# --

sub author_filter() {

  my $results  = shift;
  my $authors  = shift;

    my @filtered_results;
    my $res;

    # -- throw out case of no specified authors :
    if (scalar(@$authors) == 0) {return $results} 

    # -- we have some authors to check, so run through list of things 
    #    to filter :

    foreach $res (@$results) {
        my @matches = ();
        my $auth    = $res->{THINGY}->author;

        foreach (@$authors) {

            # -- want to match case-independent, so down-case both in the
            #    test :
            if (lc($_) eq lc($auth)) {push (@matches, $_)}
          }

        if (scalar(@matches) >= 1) {
            $res->{AUTHMATCHES} = @matches;
            push(@filtered_results, $res);
          }
      }
    return \@filtered_results;
  }


# -----------------------------------------------------------------------------
# -- filter results by time-constraints;  both bounds _must_ be set (setting
#    to 01/01/00 and 31/12/2037, respectively, effectively passes 
#    everything) :
#        
#    ...AND what about case of TIME field missing from the THINGY?


sub date_filter() {

  my $results = shift;
  my $lower   = shift;
  my $upper   = shift;

    my @filtered_results;
    my ($day, $month, $year);

    ($day, $month, $year) = split(/\//, $lower);
###    $year  = $year + 2000;
    $lower = timegm(0, 0, 0, $day, $month-1, $year-1900);

    ($day, $month, $year) = split(/\//, $upper);
###    $year  = $year + 2000;
    $upper = timegm(0, 0, 0, $day, $month-1, $year-1900);

    foreach (@$results) {
##print "\n<BR>result time: " . $_->{THINGY}->time;
        if (($_->{THINGY}->time >= $lower) && ($_->{THINGY}->time <= $upper)) {
            push(@filtered_results, $_);
          }
      }

    return \@filtered_results;
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
