#!/usr/bin/perl -w -I./_navigate_lib -I./_Doc -I./_sal -I./_cgi_lib -I./_config

use strict;

# -----------------------------------------------------------------------------
# -- LIBS :

use config;
use log;

# -----------------------------------------------------------------------------
# -- SAL :

use CDF;
use Cookie;
use Debug;
use Doc;
use Exception;
use State;

# -----------------------------------------------------------------------------
# -- ME :

use history;
use nav_mod;
use view_entries;

# -----------------------------------------------------------------------------
# -- INITIALISE :

####print "Content-type:text/html\n\n";

my $doc     = Doc->new();
my $cookies = Cookie->new($doc);
my $except  = Exception->new();
my $s       = State->new($cookies, $doc, $except);

# -- read stuff from installation configuration :

my $cgi_root_dir = &cgi_root_dir;


# -----------------------------------------------------------------------------
# -- DEBUG :

my $debug = Debug->new(1, "Important ENV variables");

$debug->item("username: " . $s->attr("USERNAME"));
$debug->item("tree: "     . $s->attr("TREE"));
$debug->item("location: " . $s->attr("LOCATION"));
$debug->wen($doc);


# -----------------------------------------------------------------------------
# -- AUTHENTICATE :

# -- we'll have a valid cookie unless this is the first navigate;  hence if
#    we don't we must be logging in, so check the uname and passwd given :

unless ($cookies->valid_cookie($doc, $except, $s->attr("USERNAME"))) {
    $doc->html_hdr("", "Navigation Widget", "");
    $doc->doc_hdr("headerstd", 
                  $s->attr("USERNAME"), $s->attr("TREE"), 
                  $s->attr("LOCATION"), "???");
    $doc->flush();
    $except->raise_logout_3($cookies, $doc, $s, "invalid_cookie");
  }


# -----------------------------------------------------------------------------
# -- DOC(HTML) BEGIN :

$doc->html_hdr("", "Navigation Widget", "");


# -----------------------------------------------------------------------------
# -- READ THINGS FROM CURRENT (GIVEN) CDF :

my $cdf = CDF->new_2($cookies, $doc, $except, $s);

my $childs          = $cdf->children;
my $an_sorted_conts = $cdf->an_sort_cts;
my $locat           = $cdf->location;
my $parent          = $cdf->parent;  
my $readme          = $cdf->readme;


# -----------------------------------------------------------------------------
# -- DOC(HEADER) BEGIN :
#
#     -- send out standard page header;
 
$doc->doc_hdr_2("headerstd", $s, $parent);
$doc->flush();


# -----------------------------------------------------------------------------
# -- ACCESS-RIGHTS/PERMISSIONS:
#
#     -- check access-rights and if none raise an exception;

unless ($cdf->read_permission_granted($except, $s)) {
    $except->raise_options($doc, $s,
                           "You don't have permission " . 
                           "to access this tree/location.");
  }


# -----------------------------------------------------------------------------
# -- OUTPUT readme, and search and upload links :
#
#     -- send out links to children of current CDF together with search and 
#        upload links :



# -----------------------------------------------------------------------------
# -- IF asked to display whole TREE, do so:
# -----------------------------------------------------------------------------

if ($s->attr("NAVIGATE") eq "VIEWTREE") {

    $doc->append(&print_tree($cookies, $doc, $except, $s, "not_admin"));
    $doc->wen();  
    exit;
  }


# -----------------------------------------------------------------------------
# -- ELSIF asked to display HISTORY of a thingy, do so :
# -----------------------------------------------------------------------------


elsif ($s->attr("NAVIGATE") eq "HISTORY") {

    my $thingy = Thingy->down_new_2($except, $s);

    $doc->append(&print_history($doc, $s, $thingy));
    $doc->wen();  
    exit;
  }


# -----------------------------------------------------------------------------
# -- ELSIF we're just navigating around, so OUTPUT ENTRIES of CDF :
# -----------------------------------------------------------------------------


elsif ($s->attr("NAVIGATE") eq "NAVIGATE") {

    $doc->append(&cdf_readme_and_links_table($readme, $childs, $s));
    $doc->append("<HR>\n");
    $doc->append("<DL>\n");


    # -- if we're in view-one-only mode, e.g., come from a link resulting from
    #    a search, show only the corresponding Entry, else show all Entries :

    # -- case of show one Entry only (e.g, a link from a search results page) :
    if ($s->attr("VIEW") eq "ONEONLY") {

        $doc->append("<DT><B><BIG>" . $s->attr("VIEWONEONLY") . 
                     "</BIG></B></DT>\n");
        $doc->append("<DD>\n");
        my $thingy = Thingy->down_new($s->attr("TREE"), $s->attr("LOCATION"),
                                      $s->attr("VIEWONEONLY"));
        $doc->append(&nav_thingy($cookies, $doc, $except, $s, $thingy));
        $doc->append("</DD>\n");
      }
  
    # -- case of show all Entries of a CDF in reverse chronological order :
    elsif ($s->attr("VIEW") eq "ALPHANUM") {
        foreach (@$an_sorted_conts) {
            $doc->append("<DT><B><BIG>$_</BIG></B></DT>\n");
            $doc->append("<DD>\n");
            my $thingy = Thingy->down_new($s->attr("TREE"), 
                                          $s->attr("LOCATION"), $_);
            $doc->append(&nav_thingy($cookies, $doc, $except, $s, $thingy));
            $doc->append("</DD>\n");
          }
      }

    # -- case of show all Entries of a CDF in reverse chronological order :
    elsif ($s->attr("VIEW") eq "REVCHRON") {

        my %thingies;
        my @th_times = ();
        my $thingy_key = "";

        foreach (@$an_sorted_conts) {
            my $thingy = Thingy->down_new($s->attr("TREE"), 
                                          $s->attr("LOCATION"), $_);

            $thingy_key = $thingy->datewrit . $thingy->time;
                # ...but catenating date/time is better as then the most 
                #    recently uploaded is shown first :-)

            $thingies{$thingy_key} = $thingy;
            push(@th_times, $thingy_key);
          }

        @th_times = sort {$b <=> $a} @th_times;

        foreach (@th_times) {
            $doc->append("<DT><B><BIG>" . $thingies{$_}->name . 
                         "</BIG></B></DT>\n");
            $doc->append("<DD>\n");
            $doc->append(&nav_thingy($cookies, $doc, $except, $s, 
                                     $thingies{$_}));
            $doc->append("</DD>\n");
          }
      }

    # -- fall back case :
    else {
        $doc->append("No view selected!");
      }


    $doc->append("</DL>\n");
    $doc->append("\n");
  }


# -----------------------------------------------------------------------------
# -- ELSE
# -----------------------------------------------------------------------------

else {
    $except->raise_options($doc, $s,
                           "You haven't asked me to do anything!");
  }


# -----------------------------------------------------------------------------
$doc->wen();
# -----------------------------------------------------------------------------





